@@ -79,14 +79,13 @@ gclue_compass_finalize (GObject *object)
         g_clear_object (&priv->cancellable);
 
         if (priv->proxy != NULL) {
-                GError *error = NULL;
+                g_autoptr(GError) error = NULL;
 
                 if (!compass_call_release_compass_sync (priv->proxy,
                                                         NULL,
                                                         &error)) {
                         g_warning ("Failed to release compass: %s",
                                    error->message);
-                        g_error_free (error);
                 }
                 g_debug ("IIO compass released");
                 g_object_unref (priv->proxy);
@@ -141,15 +140,16 @@ on_compass_claimed (GObject      *source_object,
 {
         GClueCompass *compass;
         Compass *proxy = COMPASS (source_object);
-        GError *error = NULL;
+        g_autoptr(GError) error = NULL;
 
         if (!compass_call_claim_compass_finish (proxy, res, &error)) {
-                if (error->code != G_IO_ERROR_CANCELLED)
+                if (error && !g_error_matches (error, G_IO_ERROR,
+                                               G_IO_ERROR_CANCELLED)) {
                         g_debug ("Failed to claim IIO proxy compass: %s",
                                  error->message);
-                g_error_free (error);
-                g_object_unref (proxy);
+                }
 
+                g_object_unref (proxy);
                 return;
         }
         g_debug ("IIO compass claimed");
@@ -173,21 +173,22 @@ on_compass_proxy_ready (GObject      *source_object,
 {
         GClueCompass *compass;
         Compass *proxy;
-        GError *error = NULL;
+        g_autoptr(GError) error = NULL;
 
         proxy = compass_proxy_new_for_bus_finish (res, &error);
         if (proxy == NULL) {
-                if (error->code != G_IO_ERROR_CANCELLED)
+                if (error && !g_error_matches (error, G_IO_ERROR,
+                                               G_IO_ERROR_CANCELLED)) {
                         g_debug ("Failed to connect to IIO compass proxy: %s",
                                  error->message);
-                g_error_free (error);
+                }
 
                 return;
         }
 
         compass = GCLUE_COMPASS (user_data);
 
-        compass_call_claim_compass (proxy, 
+        compass_call_claim_compass (proxy,
                                     compass->priv->cancellable,
                                     on_compass_claimed,
                                     compass);
