@@ -174,6 +174,8 @@ towertec_to_radiotype (GClueTowerTec tec,
         return TRUE;
 }
 
+#define USER_AGENT (PACKAGE_NAME "/" PACKAGE_VERSION)
+
 SoupMessage *
 gclue_mozilla_create_query (GClueMozilla  *mozilla,
                             gboolean skip_tower,
@@ -183,6 +185,7 @@ gclue_mozilla_create_query (GClueMozilla  *mozilla,
 {
         gboolean has_tower = FALSE, has_bss = FALSE;
         SoupMessage *ret = NULL;
+        SoupMessageHeaders *request_headers;
         JsonBuilder *builder;
         g_autoptr(GList) bss_list = NULL;
         JsonGenerator *generator;
@@ -255,11 +258,13 @@ gclue_mozilla_create_query (GClueMozilla  *mozilla,
                         WPABSS *bss = WPA_BSS (iter->data);
                         char mac[BSSID_STR_LEN + 1] = { 0 };
                         gint16 strength_dbm;
+                        guint age_ms;
 
                         if (gclue_mozilla_should_ignore_bss (bss))
                                 continue;
 
                         json_builder_begin_object (builder);
+
                         json_builder_set_member_name (builder, "macAddress");
                         get_bssid_from_bss (bss, mac);
                         json_builder_add_string_value (builder, mac);
@@ -267,8 +272,12 @@ gclue_mozilla_create_query (GClueMozilla  *mozilla,
                         json_builder_set_member_name (builder, "signalStrength");
                         strength_dbm = wpa_bss_get_signal (bss);
                         json_builder_add_int_value (builder, strength_dbm);
-                        json_builder_end_object (builder);
 
+                        json_builder_set_member_name (builder, "age");
+                        age_ms = 1000 * wpa_bss_get_age (bss);
+                        json_builder_add_int_value (builder, age_ms);
+
+                        json_builder_end_object (builder);
                         has_bss = TRUE;
                 }
                 json_builder_end_array (builder);
@@ -286,6 +295,8 @@ gclue_mozilla_create_query (GClueMozilla  *mozilla,
 
         uri = gclue_mozilla_get_locate_url (mozilla);
         ret = soup_message_new ("POST", uri);
+        request_headers = soup_message_get_request_headers (ret);
+        soup_message_headers_append (request_headers, "User-Agent", USER_AGENT);
         body = g_bytes_new_take (data, data_len);
         soup_message_set_request_body_from_bytes (ret, "application/json", body);
         g_debug ("Sending following request to '%s':\n%s", uri, data);
@@ -545,6 +556,7 @@ gclue_mozilla_create_submit_query (GClueMozilla  *mozilla,
 
         ret = soup_message_new ("POST", url);
         request_headers = soup_message_get_request_headers (ret);
+        soup_message_headers_append (request_headers, "User-Agent", USER_AGENT);
         if (nick != NULL && nick[0] != '\0')
                 soup_message_headers_append (request_headers,
                                              "X-Nickname",
